﻿//DESCRIPTION: Convert footnotes to dynamic sidenotes
// Peter Kahrel - www.kahrel.plus.com

#target indesign

if (parseInt (app.version) > 5 && app.documents.length > 0)
	foot_to_side (app.documents[0], "sidenote");


function foot_to_side (doc, stylename)
	{
	var side_frame, endnote, endnote_link, reference;
	var dimensions = sidenote_coordinates (doc);
	check_styles (doc, stylename, dimensions);
	if (dimensions.numbered == true)
		var cr_format = doc.crossReferenceFormats.item (stylename);
	var p_style = doc.paragraphStyles.item (stylename);
	var o_style = doc.objectStyles.item (stylename);
	var story = find_story (doc);
	var footn = story.footnotes;
	pbar____ = progress_bar (footn.length, "Sidenotes");
	for (var i = footn.length-1; i > -1; i--)
		{
		pbar____.value = i;
		// Create a frame at the footnote reference's position
		side_frame = add_frame (footn[i].storyOffset, o_style, dimensions.width);
		// Move the footnote's contents to the new frame
		endnote = footn[i].texts[0].move (LocationOptions.after, side_frame.insertionPoints[0]);
		// Apply sidenote text style
		endnote.applyParagraphStyle (p_style, false);
		// Fit the frame to its contents
		side_frame.fit (FitOptions.frameToContent);
		if (dimensions.numbered == true)
			{
			// Create cross-reference -- three steps
			endnote_link = doc.paragraphDestinations.add (endnote.insertionPoints[0]);
			reference = doc.crossReferenceSources.add (footn[i].storyOffset, cr_format);
			doc.hyperlinks.add (reference, endnote_link, {visible: false});
			}
		}
	delete_notemarkers (doc);
	if (dimensions.numbered == true)
		doc.crossReferenceSources.everyItem().update();
	}


function add_frame (ins_point, style, width)
	{
	var sidebar = ins_point.textFrames.add ();
	sidebar.geometricBounds = [0, 0, "20pt", width];
	sidebar.label = style.name;
	sidebar.appliedObjectStyle = style;
	return sidebar
	}


function find_story (doc)
	{
	app.findGrepPreferences = null;
	app.findChangeGrepOptions.includeFootnotes = false;
	app.findGrepPreferences.findWhat = '~F';
	var fn = doc.findGrep ();
	if (fn.length > 0)
		return fn[0].parentStory;
	else
		exit ();
	}


function delete_notemarkers (doc)
	{
	app.findGrepPreferences = app.changeGrepPreferences = null;
	app.findChangeGrepOptions.includeFootnotes = true;
	app.findGrepPreferences.findWhat = '~F';
	doc.changeGrep ();
	}


function check_styles (doc, s, sidenote)
	{
	text_styles (doc, s, sidenote);
	footnote_options (doc);
	if (doc.objectStyles.item (s) == null)
		{
		with (doc.objectStyles.add ({name: s}))
			{
			// Store the width of the frame so we can use that later when
			// adding notes semi-manually
			label = sidenote.width;
			basedOn = doc.objectStyles[0];
			enableParagraphStyle = true;
			appliedParagraphStyle = s;
			enableStroke = true;
			strokeWeight = 0;
//~ 			enableFill = true;
//~ 			fillColor = 'Black';
//~ 			fillTint = 15;
			enableAnchoredObjectOptions = true;
			with (anchoredObjectSettings)
				{
				spineRelative = true;
				anchoredPosition = AnchorPosition.anchored;
				anchorPoint = AnchorPoint.topRightAnchor;
				horizontalReferencePoint = AnchoredRelativeTo.columnEdge;
				anchorXoffset = sidenote.gutter;
				horizontalAlignment = HorizontalAlignment.leftAlign;
				verticalReferencePoint = VerticallyRelativeTo.capheight;
				pinPosition = true;
				}
			}
		}
	}


function text_styles (doc, n, sidenote)
	{
	try
		{
		var ps = doc.paragraphStyles.add ({name: n});
		ps.basedOn = doc.paragraphStyles[1];
		//ps.justification = Justification.leftAlign;
		ps.justification = Justification.toBindingSide;
		ps.pointSize = 9;
		} catch (e) {errorM (e.message)};
		
	if (sidenote.numbered == true)
		{
		try
			{
			var numlist = doc.numberingLists.add ({name: n});
			numlist.continueNumbersAcrossStories = true;
			numlist.continueNumbersAcrossDocuments = true;
			} catch (e) {errorM (e.message)};
		try
			{
			var char_style = doc.characterStyles.add ({name: n + "_reference"});
			char_style.position = Position.superscript;
			} catch (e) {errorM (e.message)};
		try
			{
			var char_style = doc.characterStyles.add ({name: n + "_number"});
			} catch (e) {errorM (e.message)};
		try
			{
			var cr = doc.crossReferenceFormats.add ({name: n});
			cr.appliedCharacterStyle = doc.characterStyles.item (n + "_reference");
			cr.buildingBlocks.add (BuildingBlockTypes.paragraphNumberBuildingBlock);
			} catch (e) {errorM (e.message)};
		try
			{
			ps.appliedNumberingList = numlist;
			ps.bulletsAndNumberingListType = ListType.numberedList;
			ps.tabList = [{position: "12pt", alignment: TabStopAlignment.leftAlign}];
			ps.numberingCharacterStyle = doc.characterStyles.item (n + "_number");
			} catch (e) {errorM (e.message)};
		}
	}


function footnote_options (doc)
	{
	doc.footnoteOptions.separatorText = "";
	}

// Interface =================================================================

function sidenote_coordinates (doc)
	{
	var w = new Window ('dialog', 'Sidenotes');
	var doc_unit = doc_units ();
	w.alignChildren = ['right', 'top'];
		var main = w.add ('panel');
		main.alignChildren = ['right', 'top'];
		main.orientation = 'column';
			var g1 = main.add ('group');
				g1.add ('statictext', undefined, 'Ширина фрейма: ');
				var width = g1.add ('edittext');
				width.characters = 10;
				width.text = convert_units ("70 pt", doc_unit);
				width.active = true;
			var g2 = main.add ('group');
				g2.add ('statictext', undefined, 'Расстояние до текста: ');
				var space = g2.add ('edittext');
				space.text = convert_units ("12 pt", doc_unit);
				space.characters = 10;
		var numbered = w.add ('checkbox', undefined, "Нумерация сносок\u00A0");
		numbered.value = true;
		numbered.alignment = 'left';
		var buttons = w.add ('group');
			buttons.orientation = 'row';
			buttons.alignChildren = ['right', 'bottom'];
			buttons.add ('button', undefined, 'OK');
			buttons.add ('button', undefined, 'Cancel', {name: 'cancel'});

		width.onChange = function () {width.text = convert_units (width.text, doc_unit)};
		space.onChange = function () {space.text = convert_units (space.text, doc_unit)};
		
	if (w.show () == 2)
		exit ();
	return {width: width.text, gutter: space.text, numbered: numbered.value}
	}



function convert_units (n, to)
	{
	var m = [];
	m["ag"] = 5.1428571428;
	m["p"] = 12.0000000000;
	m["mm"] = 2.8346456692;
	m["cm"] = 28.3464566929;
	m["in"] = 72.00000000;
	m["c"] = 12.7878751998;
	m["tr"] = 3.0112500000; // traditional point -- but we don't do anything with it yet
	m["pt"] = 1.0000000000;
	obj = fix_measurement (n);
	var temp = (obj.amount * m[obj.unit]) / m[to];
	return output_format (temp, to)
	}


// Add the target unit to the amount, either suffixed pt, ag, mm, cm, in,
// or infixed p or c

function output_format (amount, target)
	{
	amount = amount.toFixed (3).replace (/\.?0+$/g, "");
	if (target.length == 2) // two-character unit: pt, mm, etc.
		return String (amount) + " " + target;
	else // "p" or "c"
		{
		// calculate the decimal
		var decimal = (Number (amount) - parseInt (amount)) * 12;
		// return the integer part of the result + infix + formatted decimal
		return parseInt (amount) + target + decimal;
		}
	}


function fix_measurement (n)
	{
	// infixed "p" and "c" to decimal suffixes: 3p4 > 3.5 p
	n = n.replace (/(\d+)([pc])([.\d]+)$/, function () {return Number (arguments[1]) + Number (arguments[3]/12) + arguments[2]});
	// add unit if necessary
	n = n.replace (/(\d)$/, "$1" + doc_units (app.documents[0]))
	// split on unit
	var temp = n.split (/(ag|cm|mm|c|pt|p|in)$/);
	if (temp.length == 1)
		return {amount: Number (temp[0]), unit: doc_units ()};
	else
		return {amount: Number (temp[0]), unit: temp[1]};
	}


function doc_units ()
	{
	switch (app.documents[0].viewPreferences.horizontalMeasurementUnits)
		{
		case 2051106676: return 'ag';
		case 2053336435: return 'cm';
		case 2053335395: return 'c';
		case 2053729891: return 'in';
		case 2053729892: return 'in';
		case 2053991795: return 'mm';
		case 2054187363: return 'p';
		case 2054188905: return 'pt';
		}
	}


function errorM (m)
	{
	alert (m, "Error", true)
	exit ()
	}


function progress_bar (stop, title)
	{
	progressw = new Window ('palette', title);
	pb____ = progressw.add ('progressbar', undefined, 0, stop);
	pb____.preferredSize = [300,20];
	progressw.show()
	return pb____;
	}